#!/usr/bin/perl

# $Id: create-db.pl,v 1.3 2002/01/04 15:54:42 smoonen Exp $

if(!(-t))
  { die "You must execute this script from the command line.\n"; }

# This script is used to create the initial database for WikkiTikkiTavi
# to use for page storage.

if($#ARGV < 2)
{
  print "Usage: \n";
  print "  perl ./create-db.pl dbname dbuser dbpassword [table_prefix] [dbserver] [dbtype]\n";
  print "\n";
  print "Examples:\n\n";
  print "  perl ./create-db.pl wiki joe passwd\n";
  print "  perl ./create-db.pl project sally pass wiki_ database.example.com\n";
  print "  perl ./create-db.pl common jim key \"\" database.example.com\n";
  print "  perl ./create-db.pl vemshu_vemstavi vemshu_tavi secret efi_ localhost postgresql\n";
  exit;
}

$database = $ARGV[0];                   # Database name.
$user     = $ARGV[1];                   # Database user name.
$pass     = $ARGV[2];                   # Database password.
if($#ARGV > 2)                          # Table prefix.
  { $prefix = $ARGV[3]; }
else
  { $prefix = ""; }
if($#ARGV > 3)                          # Database host.
  { $dbhost = $ARGV[4]; }
else
  { $dbhost = ""; }
if($#ARGV > 4)				# Database type.
  { $dbtype = $ARGV[5]; }
else
  { $dbtype = ""; }

use DBI;

print "Connecting to database...\n";

if ($dbtype == "postgresql") {
	$port = "5432";			# This could be an option later
	$dbh = DBI->connect("DBI:Pg:dbname=$database;host=$dbhost;port=$port", $user, $pass)
		or die "Connecting: $DBI::errstr\n";
} else {
	$dbh = DBI->connect("DBI:mysql:$database:$dbhost", $user, $pass)
       		or die "Connecting: $DBI::errstr\n";
};

# end here for now

print "Creating database tables...\n";

if ($dbtype == "postgresql") {
	
	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "links ( "
			     . "page VARCHAR(80) DEFAULT '' NOT NULL, "
			     . "link VARCHAR(80) DEFAULT '' NOT NULL, "
			     . "count INTEGER DEFAULT '0' NOT NULL, "
			     . "PRIMARY KEY (page, link) )");
	$qid->execute or die "Error creating " .$prefix . "links table.\n";	

	$qid->$dbh->prepare("create index " . $prefix . "links_link on " 
			     . $prefix . "links(link)");
	$qid->execute or die "Error creating " .$prefix . "links_link index.\n";	

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "pages ( "
			     . "title VARCHAR(80) DEFAULT '' NOT NULL, "
			     . "version INTEGER DEFAULT '1' NOT NULL, "
			     . "time TIMESTAMP, "
			     . "supercede TIMESTAMP, "
			     . "mutable VARCHAR(3) CHECK (mutable = 'on' OR mutable = 'off') NOT NULL DEFAULT 'on', "
			     . "username VARCHAR(80) default NULL, "
 			     . "author VARCHAR(80) NOT NULL, "
			     . "comment VARCHAR(80) NOT NULL, "
			     . "body TEXT, "
			     . "PRIMARY KEY (title,version) )");
	$qid->execute or die "Error creating " .$prefix . "pages table.\n";	

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "rate ( "
			     . "ip CIDR NOT NULL, "
			     . "time TIMESTAMP, "
			     . "viewLimit SMALLINT, "
			     . "searchLimit SMALLINT, "
			     . "editLimit SMALLINT, "
			     . "PRIMARY KEY (ip) )");
	$qid->execute or die "Error creating " .$prefix . "rate table.\n";	

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "interwiki ( "
			     . "prefix VARCHAR(80) NOT NULL, "
 			     . "where_defined VARCHAR(80) NOT NULL, "
 			     . "url VARCHAR(255) NOT NULL, "
			     . "PRIMARY KEY (prefix) )");
	$qid->execute or die "Error creating " .$prefix . "interwiki table.\n";	

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "sisterwiki ( "
			     . "prefix VARCHAR(80) NOT NULL, "
			     . "where_defined VARCHAR(80) NOT NULL, "
			     . "url VARCHAR(255) NOT NULL, "
			     . "PRIMARY KEY (prefix) )");
	$qid->execute or die "Error creating " .$prefix . "sisterwiki table.\n";	

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "remote_pages ( "
			     . "page VARCHAR(80) NOT NULL, "
			     . "site VARCHAR(80) NOT NULL, "
			     . "PRIMARY KEY (page,site) )");
	$qid->execute or die "Error creating " .$prefix . "remote_pages table.\n";	

} else {

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "links ( "
        	             . "page varchar(80) DEFAULT '' NOT NULL, "
                	     . "link varchar(80) DEFAULT '' NOT NULL, "
	                     . "count int(10) unsigned DEFAULT '0' NOT NULL, "
        	             . "PRIMARY KEY (page, link) )");
	$qid->execute or die "Error creating table\n";

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "pages ( "
        	             . "title varchar(80) DEFAULT '' NOT NULL, "
	                     . "version int(10) unsigned DEFAULT '1' NOT NULL, "
        	             . "time timestamp(14), "
                	     . "supercede timestamp(14), "
	                     . "mutable set('off', 'on') DEFAULT 'on' NOT NULL, "
        	             . "username varchar(80), "
                	     . "author varchar(80) DEFAULT '' NOT NULL, "
	                     . "comment varchar(80) DEFAULT '' NOT NULL, "
        	             . "body text, "
                	     . "PRIMARY KEY (title, version) )");
	$qid->execute or die "Error creating table\n";

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "rate ( "
        	             . "ip char(20) DEFAULT '' NOT NULL, "
	                     . "time timestamp(14), "
        	             . "viewLimit smallint(5) unsigned, "
                	     . "searchLimit smallint(5) unsigned, "
	                     . "editLimit smallint(5) unsigned, "
        	             . "PRIMARY KEY (ip) )");
	$qid->execute or die "Error creating table\n";

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "interwiki ( "
        	             . "prefix varchar(80) DEFAULT '' NOT NULL, "
                	     . "where_defined varchar(80) DEFAULT '' NOT NULL, "
	                     . "url varchar(255) DEFAULT '' NOT NULL, "
        	             . "PRIMARY KEY (prefix ) )");
	$qid->execute or die "Error creating table\n";

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "sisterwiki ( "
        	             . "prefix varchar(80) DEFAULT '' NOT NULL, "
                	     . "where_defined varchar(80) DEFAULT '' NOT NULL, "
	                     . "url varchar(255) DEFAULT '' NOT NULL, "
        	             . "PRIMARY KEY (prefix ) )");
	$qid->execute or die "Error creating table\n";

	$qid = $dbh->prepare("CREATE TABLE " . $prefix . "remote_pages ( "
        	             . "page varchar(80) DEFAULT '' NOT NULL, "
                	     . "site varchar(80) DEFAULT '' NOT NULL, "
	                     . "PRIMARY KEY (page, site) )");
	$qid->execute or die "Error creating table\n";

};

print "\n";
print "Your tables were created.  Next you should run configure.pl\n";
print "to configure your preferences.\n";

if ($dbtype == "postgresql") {
 print "\n";
 print "If you wish to transfer data from MySQL 4.1 to PostgreSQL 7.4,\n";
 print "you should run transfer-mysql41-to-postgresql74.sh\n"
};

